/*******************************************************************************
* File Name: My_CapSense_1_INT.c
* Version 2.10
*
* Description:
*  This file provides the source code of the  Interrupt Service Routine (ISR)
*  for the CapSense CSD component.
*
* Note:
*
********************************************************************************
* Copyright 2014, Cypress Semiconductor Corporation.  All rights reserved.
* You may use this file only in accordance with the license, terms, conditions,
* disclaimers, and limitations in the end user license agreement accompanying
* the software package with which this file was provided.
*******************************************************************************/

#include "My_CapSense_1.h"
#include "My_CapSense_1_PVT.h"

/*******************************************************************************
*  Place your includes, defines and code here 
********************************************************************************/
/* `#START My_CapSense_1_ISR_INTC` */

/* `#END` */


/*******************************************************************************
* Function Name: My_CapSense_1_ISR
********************************************************************************
*
* Summary:
*  This ISR is executed when the measure window is closed. The measure window 
*  depends on the sensor scan resolution parameter. 
*  The ISR has two modes: 
*   1. Scans a single sensor and stores the measure result.
*   2. Stores the result of the current measure and starts the next scan till all 
*      the enabled sensors are scanned.
*  This interrupt handler is only used in one channel designs.
*
* Parameters:
*  None
*
* Return:
*  None
*
* Global Variables:
*  My_CapSense_1_csv - used to provide the status and mode of the scanning process.
*   Mode - a single scan or scan of all the enabled sensors.
*   Status - scan is in progress or ready for new scan.
*  My_CapSense_1_sensorIndex - used to store a sensor scanning sensor number.
*
* Reentrant:
*  No
*
*******************************************************************************/
CY_ISR(My_CapSense_1_ISR)
{
    static uint8 My_CapSense_1_snsIndexTmp;
	/*  Place your Interrupt code here. */
    /* `#START My_CapSense_1_ISR_ENTER` */

    /* `#END` */

	CyIntDisable(My_CapSense_1_ISR_NUMBER);
    
    My_CapSense_1_CSD_INTR_REG = 1u;

    My_CapSense_1_PostScan((uint32)My_CapSense_1_sensorIndex);
            
    if ((My_CapSense_1_csdStatusVar & My_CapSense_1_SW_CTRL_SINGLE_SCAN) != 0u)
    {
        My_CapSense_1_csdStatusVar &= (uint8)~My_CapSense_1_SW_STS_BUSY;
    }
    else
    {
        /* Proceed scanning */
        My_CapSense_1_sensorIndex = My_CapSense_1_FindNextSensor(My_CapSense_1_sensorIndex);
        
        /* Check end of scan */
        if(My_CapSense_1_sensorIndex < My_CapSense_1_TOTAL_SENSOR_COUNT)
        {
            My_CapSense_1_PreScan((uint32)My_CapSense_1_sensorIndex);
        }
        else
        {
            My_CapSense_1_csdStatusVar &= (uint8)~My_CapSense_1_SW_STS_BUSY;

			 /* Check if widget has been scanned */
			if((My_CapSense_1_csdStatusVar & My_CapSense_1_SW_CTRL_WIDGET_SCAN) != 0u)
			{
			    /* Restore sensorEnableMask array */
				for(My_CapSense_1_snsIndexTmp = 0u; 
				    My_CapSense_1_snsIndexTmp < My_CapSense_1_TOTAL_SENSOR_MASK; 
					My_CapSense_1_snsIndexTmp++)
				{
					/* Backup sensorEnableMask array */
					My_CapSense_1_sensorEnableMask[My_CapSense_1_snsIndexTmp] = My_CapSense_1_sensorEnableMaskBackup[My_CapSense_1_snsIndexTmp];
				}
			}
        } 
    }
    CyIntEnable(My_CapSense_1_ISR_NUMBER);
	
	/*  Place your Interrupt code here. */
    /* `#START My_CapSense_1_ISR_EXIT` */

    /* `#END` */
}

/* [] END OF FILE */
