/*******************************************************************************
* File Name: My_CapSense_1_PVT.h
* Version 2.10
*
* Description:
*  This file provides constants and structure declarations for the
*  CapSense CSD component.
*
* Note:
*
********************************************************************************
* Copyright 2014, Cypress Semiconductor Corporation.  All rights reserved.
* You may use this file only in accordance with the license, terms, conditions,
* disclaimers, and limitations in the end user license agreement accompanying
* the software package with which this file was provided.
*******************************************************************************/


#if !defined(CY_CAPSENSE_CSD_PVT_My_CapSense_1_H)
#define CY_CAPSENSE_CSD_PVT_My_CapSense_1_H

#include "My_CapSense_1.h"

#if(0u != My_CapSense_1_CSHL_API_GENERATE)
	#include "My_CapSense_1_CSHL.h"
#endif /* (0u != My_CapSense_1_CSHL_API_GENERATE) */

#if(My_CapSense_1_TOTAL_TRACKPAD_GESTURES_COUNT > 0u)
	#include "My_CapSense_1_GESTURE.h"
#endif /* (My_CapSense_1_TOTAL_TRACKPAD_GESTURES_COUNT > 0u) */
	
typedef struct
{
	#if (My_CapSense_1_IS_COMPLEX_SCANSLOTS)
	uint8 sensor;
	#endif  /* My_CapSense_1_IS_COMPLEX_SCANSLOTS */
	uint8 pinNumber;
	uint8 portNumber;
	uint8 idacValue;
	uint16 senseClkDiv;
	uint16 sampleClkDiv;
	uint8 avgSamplesNum;
	uint16 calibrationCount;
	uint32 scanResolution;
}My_CapSense_1_CP_MEASURE_CONFIG;

typedef struct
{
    uint8 *idac1LevelsTbl;
    uint8 *idac2LevelsTbl;
    uint8 *sensetivityTbl;
    uint8 *chargeDivTbl;
    uint8 *modDivTbl;
    const uint8  *widgetNubmersTbl;
    uint32 *resolutionsTbl;
	uint8 const  *rawDataIndex;
	uint8 const *numberOfSensors;
    uint16 (*ptrGetRaw)(uint32 sensor);
	void (*ptrPreScan)(uint32 sensor);
	uint16 (*ptrReadSensorRaw)(uint32 sensor);
    uint32 (*ptrGetBitValue)(const uint8 table[], uint32 position);
	void (*ptrSetBitValue)(uint8 table[], uint32 position, uint32 value);
}My_CapSense_1_CONFIG_TYPE_POINTERS_P4_v2_10;

typedef struct
{
    uint8 totalScanslotsNum;
	uint8 totalWidgetsNum;
    uint8 totalSensorsNum;
	uint16 sensetivitySeed;
	uint16 hfclkFreq_MHz;
    My_CapSense_1_CONFIG_TYPE_POINTERS_P4_v2_10 const * pointers;
}My_CapSense_1_CONFIG_TYPE_P4_v2_10;

#define My_CapSense_1_AVG_SAMPLES_NUM		(1u)


/***************************************
*        Function Prototypes
***************************************/
void My_CapSense_1_PreScan(uint32 sensor);
void My_CapSense_1_PostScan(uint32 sensor);

__INLINE void My_CapSense_1_EnableSensor(uint32 sensor);
__INLINE void My_CapSense_1_DisableSensor(uint32 sensor);

#if (My_CapSense_1_IS_COMPLEX_SCANSLOTS)
    void My_CapSense_1_EnableScanSlot(uint32 slot);
    void My_CapSense_1_DisableScanSlot(uint32 slot);
#else
    #define My_CapSense_1_EnableScanSlot(slot)   My_CapSense_1_EnableSensor(slot)
    #define My_CapSense_1_DisableScanSlot(slot)  My_CapSense_1_DisableSensor(slot)
#endif  /* End My_CapSense_1_IS_COMPLEX_SCANSLOTS */

void My_CapSense_1_EnableShieldElectrode(uint32 pinNumber, uint32 portNumber);

uint32 My_CapSense_1_GetBitValue(const uint8 table[], uint32 position);
void My_CapSense_1_SetBitValue(uint8 table[], uint32 position, uint32 value);

/* Find next sensor for One Channel design */
uint8 My_CapSense_1_FindNextSensor(uint8 snsIndex);

void My_CapSense_1_SetPinDriveMode(uint32 driveMode, uint32 pinNumber, uint32 portNumber);
void My_CapSense_1_CalibrateSensor(uint32 sensor, uint32 rawLevel, uint8 idacLevelsTbl[]);
uint16  My_CapSense_1_GetSensorRaw(uint32 sensor);

void My_CapSense_1_DisableBaselineIDAC(void);
void My_CapSense_1_EnableBaselineIDAC(const My_CapSense_1_CONFIG_TYPE_P4_v2_10 *config);

uint32 My_CapSense_1_GetLowCp(My_CapSense_1_CP_MEASURE_CONFIG *config);
uint32 My_CapSense_1_GetHighCp(const My_CapSense_1_CP_MEASURE_CONFIG *config);
uint32 My_CapSense_1_CalibrateIDAC(My_CapSense_1_CP_MEASURE_CONFIG *config);
uint16 My_CapSense_1_ScanSensorCp(const My_CapSense_1_CP_MEASURE_CONFIG *config);

void My_CapSense_1_ConnectElectrode(uint32 pinNumber, uint32 portNumber);
void My_CapSense_1_DisconnectElectrode(uint32 pinNumber, uint32 portNumber);

void TunePrescalers_P4_v2_10(My_CapSense_1_CONFIG_TYPE_P4_v2_10 *config);
void TuneSensitivity_P4_v2_10(My_CapSense_1_CONFIG_TYPE_P4_v2_10 *config);
void MeasureNoiseEnvelope_P4_v2_10(My_CapSense_1_CONFIG_TYPE_P4_v2_10 *config, const uint8 sensorNumber, const uint16 *rawData);
void CalculateThresholds_P4_v2_10(My_CapSense_1_CONFIG_TYPE_P4_v2_10 *config, const uint8 sensorNumber, const uint16 *rawData, const uint8 widget, uint8 *fingerThres, uint8 *noiseThres);
void CalibrateSensors_P4_v2_10(My_CapSense_1_CONFIG_TYPE_P4_v2_10 *config, uint16 rawLevel);
void CalibrateOneDAC_P4_v2_10(My_CapSense_1_CONFIG_TYPE_P4_v2_10 *config, uint16 rawLevel, uint32 curSensor, uint8 *idacLevelsTbl);


#endif /* CY_CAPSENSE_CSD_PVT_My_CapSense_1_H */
