/*******************************************************************************
* File Name: My_CapSense_1_SMS.c
* Version 2.10
*
* Description:
*  This file provides the source code of wrapper between CapSense CSD component 
*  and Auto Tuning library.
*
* Note:
*
********************************************************************************
* Copyright 2014, Cypress Semiconductor Corporation.  All rights reserved.
* You may use this file only in accordance with the license, terms, conditions,
* disclaimers, and limitations in the end user license agreement accompanying
* the software package with which this file was provided.
*******************************************************************************/

#include "My_CapSense_1_SMS.h"
#include "My_CapSense_1_PVT.h"

#include "cytypes.h"

#if (My_CapSense_1_TUNING_METHOD == My_CapSense_1__TUNING_AUTO)


/*  Real Charge Divider values */
#if (CY_PSOC4_4200BL)
    /*  Dividers are not chained */
	#if (CYDEV_BCLK__HFCLK__MHZ > 24u)
		const uint8 prescalerTable[My_CapSense_1_PRESCALERS_TBL_SIZE] = {
			4u,     4u,     4u,    8u,    8u,    8u,    8u,    16u,    16u,    16u,    16u,    16u,    16u,    16u,    16u,    16u
		};
	#elif (CYDEV_BCLK__HFCLK__MHZ >12u)
		const uint8 prescalerTable[My_CapSense_1_PRESCALERS_TBL_SIZE] = {
			2u,     2u,     2u,    4u,    4u,    4u,    4u,    8u,    8u,    8u,    8u,    8u,    8u,    8u,    8u,    8u
		};
	#else   /* (CYDEV_BCLK__HFCLK__MHZ > 12u) */
		const uint8 prescalerTable[My_CapSense_1_PRESCALERS_TBL_SIZE] = {
			1u,     1u,     1u,    2u,    2u,    2u,    2u,    4u,    4u,    4u,    4u,    4u,    4u,    4u,    4u,    4u
		};
	#endif /* (CYDEV_BCLK__HFCLK__MHZ > 24u) */
	
#elif (0u != My_CapSense_1_IS_M0S8PERI_BLOCK)
    /*  Dividers are not chained */
	#if (CYDEV_BCLK__HFCLK__MHZ >= 12u)
		const uint8 prescalerTable[My_CapSense_1_PRESCALERS_TBL_SIZE] = {
			1u,     1u,     1u,    2u,    2u,    2u,    2u,    4u,    4u,    4u,    4u,    4u,    4u,    4u,    4u,    4u
		};

	#elif (CYDEV_BCLK__HFCLK__MHZ >= 6u)
		const uint8 prescalerTable[My_CapSense_1_PRESCALERS_TBL_SIZE] = {
			1u,     1u,     1u,    1u,    1u,    1u,    1u,    2u,    2u,    2u,    2u,    2u,    2u,    2u,    2u,    2u
		};
	#else
		const uint8 prescalerTable[My_CapSense_1_PRESCALERS_TBL_SIZE] = {
			1u,     1u,     1u,    1u,    1u,    1u,    1u,    1u,    1u,    1u,    1u,    1u,    1u,    1u,    1u,    1u
		};
	#endif /* (CYDEV_BCLK__HFCLK__MHZ > 12u) */

#else
    /*  Dividers are chained */ 
	#if (CYDEV_BCLK__HFCLK__MHZ > 24u)
		const uint8 prescalerTable[My_CapSense_1_PRESCALERS_TBL_SIZE] = {
			2u,     2u,     2u,    4u,    4u,    4u,    4u,    8u,    8u,    8u,    8u,    8u,    8u,    8u,    8u,    8u
		};

	#elif (CYDEV_BCLK__HFCLK__MHZ >12u)
		const uint8 prescalerTable[My_CapSense_1_PRESCALERS_TBL_SIZE] = {
			1u,     1u,     1u,    2u,    2u,    2u,    2u,    4u,    4u,    4u,    4u,    4u,    4u,    4u,    4u,    4u
		};
	#else   /* (CYDEV_BCLK__HFCLK__MHZ > 12u) */
		const uint8 prescalerTable[My_CapSense_1_PRESCALERS_TBL_SIZE] = {
			1u,     1u,     1u,    1u,    1u,    1u,    1u,    2u,    2u,    2u,    2u,    2u,    2u,    2u,    2u,    2u
		};
	#endif /* (CYDEV_BCLK__HFCLK__MHZ > 24u) */
#endif /* (0u != My_CapSense_1_IS_M0S8PERI_BLOCK) */


uint8 noiseEnvelopeTbl[My_CapSense_1_TOTAL_SENSOR_COUNT];
uint8 runningDifferenceTbl[My_CapSense_1_TOTAL_SENSOR_COUNT];
uint8 signRegisterTbl[My_CapSense_1_TOTAL_SENSOR_COUNT];
uint16 sampleMinTbl[My_CapSense_1_TOTAL_SENSOR_COUNT];
uint16 sampleMaxTbl[My_CapSense_1_TOTAL_SENSOR_COUNT];
uint16 previousSampleTbl[My_CapSense_1_TOTAL_SENSOR_COUNT];
uint8 kValueTbl[My_CapSense_1_TOTAL_SENSOR_COUNT];

uint8 SMS_LIB_SensorSensitivity[] = {
    2u, 
};

My_CapSense_1_CONFIG_TYPE_P4_v2_10 My_CapSense_1_config;
My_CapSense_1_CONFIG_TYPE_POINTERS_P4_v2_10 const My_CapSense_1_configPointers = {
    My_CapSense_1_modulationIDAC,
    My_CapSense_1_compensationIDAC,
     SMS_LIB_SensorSensitivity,
    My_CapSense_1_senseClkDividerVal,
    My_CapSense_1_sampleClkDividerVal,
    My_CapSense_1_widgetNumber,
    My_CapSense_1_widgetResolution,
	(const uint8  *)My_CapSense_1_rawDataIndex,
	(const uint8  *)My_CapSense_1_numberOfSensors,
    &My_CapSense_1_GetSensorRaw,
    &My_CapSense_1_PreScan,
    &My_CapSense_1_ReadSensorRaw,
    &My_CapSense_1_GetBitValue,
	&My_CapSense_1_SetBitValue
};


/*******************************************************************************
* Function Name: My_CapSense_1_AutoTune
********************************************************************************
*
* Summary:
*  Provides the tuning procedure for all sensors.
*
* Parameters:
*  None
*
* Return:
*  None
*
* Global Variables:
*   My_CapSense_1_config: the structure which contains configuration parameters
*   My_CapSense_1_senseClkDividerVal[]: used to store the Analog Switch 
*   dividers for each sensor.
*   My_CapSense_1_senseClkDividerVal[]: used to store the 
*    Analog Switch divider for all sensors.
*   My_CapSense_1_prescalersTuningDone - used to notify the Tuner GUI that 
*   the pre-scalers tuning  is done.
*
* Side Effects:
*  None
*
*******************************************************************************/
void My_CapSense_1_AutoTune(void)
{
	uint32 curSensor;
    uint32 widget;

	My_CapSense_1_config.totalScanslotsNum = My_CapSense_1_TOTAL_SCANSLOT_COUNT;
	My_CapSense_1_config.totalWidgetsNum = My_CapSense_1_END_OF_WIDGETS_INDEX;
	My_CapSense_1_config.totalSensorsNum = My_CapSense_1_TOTAL_SENSOR_COUNT;
	
	My_CapSense_1_config.hfclkFreq_MHz = CYDEV_BCLK__HFCLK__MHZ;
	My_CapSense_1_config.sensetivitySeed = My_CapSense_1_SENSETIVITY_FACTOR;
	
    My_CapSense_1_config.pointers = &My_CapSense_1_configPointers;       
    
	My_CapSense_1_SetAnalogSwitchesSrcDirect();
    
    for(widget = 0u; widget < My_CapSense_1_config.totalWidgetsNum; widget++)
    {
        My_CapSense_1_widgetResolution[widget] = My_CapSense_1_CALIBRATION_RESOLUTION;
    }
	

	for(curSensor = 0u; curSensor < My_CapSense_1_TOTAL_SCANSLOT_COUNT; curSensor++)
    {
		My_CapSense_1_senseClkDividerVal[curSensor] = My_CapSense_1_CALIBRATION_ASD;
		My_CapSense_1_sampleClkDividerVal[curSensor] = My_CapSense_1_CALIBRATION_MD;
	}
    
    My_CapSense_1_DisableBaselineIDAC();

    CalibrateSensors_P4_v2_10(&My_CapSense_1_config, My_CapSense_1_CAL_RAW_COUNT);

	#if(0u != My_CapSense_1_TOTAL_CENTROIDS_COUNT) 
		My_CapSense_1_NormalizeWidgets(My_CapSense_1_END_OF_WIDGETS_INDEX, My_CapSense_1_modulationIDAC);
	#endif /* (0u != My_CapSense_1_TOTAL_CENTROIDS_COUNT)  */
	
    TunePrescalers_P4_v2_10(&My_CapSense_1_config);
	
	#if(My_CapSense_1_PRS_OPTIONS != My_CapSense_1__PRS_NONE)
		My_CapSense_1_prescalersTuningDone = 1u;
	#endif /* (My_CapSense_1_PRS_OPTIONS == My_CapSense_1__PRS_NONE) */
       
    CalibrateSensors_P4_v2_10(&My_CapSense_1_config, My_CapSense_1_CAL_RAW_COUNT);
	#if(0u != My_CapSense_1_TOTAL_CENTROIDS_COUNT)
		My_CapSense_1_NormalizeWidgets(My_CapSense_1_END_OF_WIDGETS_INDEX, My_CapSense_1_modulationIDAC);
	#endif /* (0u != My_CapSense_1_TOTAL_CENTROIDS_COUNT)  */
    TuneSensitivity_P4_v2_10(&My_CapSense_1_config);
	
	My_CapSense_1_EnableBaselineIDAC(&My_CapSense_1_config);
    
    for(curSensor = 0u; curSensor < My_CapSense_1_TOTAL_SCANSLOT_COUNT; curSensor++)
    {
        My_CapSense_1_UpdateThresholds(curSensor);
    }
}


/*******************************************************************************
* Function Name: My_CapSense_1_UpdateThresholds
********************************************************************************
*
* Summary:
*  The API updates the Finger Threshold, Hysteresis, Noise Threshold, and 
*  Negative Noise Threshold in the AutoTuning (Smartsense) Mode.
*
* Parameters:
*  sensor: sensor number.
*
* Return:
*  None
*
* Global Variables:
*  My_CapSense_1_widgetNumber[] - contains widget Number for given sensor.
*  0 through 7 (sensor 0 is bit 0, sensor 1 is bit 1).
*  For other sensors the array element equals to 255. 
*  My_CapSense_1_numberOfSensors[widget] - Number of sensors in the widget.
*   My_CapSense_1_fingerThreshold[] - contains the level of signal for each sensor
*   that determines if a finger present on the sensor.
*   My_CapSense_1_negativeNoiseThreshold[] - negative Noise Threshold
*   My_CapSense_1_hysteresis[] - the array with hysteresis values.
*
* Side Effects:
*  None
*
*******************************************************************************/
void My_CapSense_1_UpdateThresholds(uint32 sensor)
{
	uint8 fingerThreshold;
	uint32 widget;
	
	widget = My_CapSense_1_widgetNumber[sensor];

	#if(0u != My_CapSense_1_FLEXIBLE_THRESHOLDS_EN)
	    fingerThreshold = My_CapSense_1_fingerThreshold[widget];
        /* Update Threshold manually (flexible) */   
		My_CapSense_1_noiseThreshold[widget] = (uint8)(fingerThreshold >> 1u);
		My_CapSense_1_negativeNoiseThreshold[widget] = (uint8)(fingerThreshold >> 1u);
		My_CapSense_1_hysteresis[widget] = (uint8)(fingerThreshold >> 3u);
	#else
		/* Calculate Finger Threshold and Noise Threshold with Smartsense (automatic) */ 	
		CalculateThresholds_P4_v2_10(&My_CapSense_1_config, (uint8)sensor, My_CapSense_1_sensorRaw, (uint8)widget, My_CapSense_1_fingerThreshold, My_CapSense_1_noiseThreshold);
		
	    fingerThreshold = My_CapSense_1_fingerThreshold[widget];
		
	    /* Update Threshold based on calculated with Smartsense (automatic) */
		My_CapSense_1_negativeNoiseThreshold[widget] = My_CapSense_1_noiseThreshold[widget];
        
		if(fingerThreshold < My_CapSense_1_THRESHOLD_1)
		{
			My_CapSense_1_hysteresis[widget] = (uint8)(fingerThreshold >> 3u);
		}
		else if(fingerThreshold <  My_CapSense_1_THRESHOLD_2)
		{
			My_CapSense_1_hysteresis[widget] = (uint8)(fingerThreshold >> 4u);
		}
		else if(fingerThreshold <  My_CapSense_1_THRESHOLD_3)
		{
			My_CapSense_1_hysteresis[widget] = (uint8)(fingerThreshold >> 5u);
		}
		else if(fingerThreshold <  My_CapSense_1_THRESHOLD_4)
		{
			My_CapSense_1_hysteresis[widget] = (uint8)(fingerThreshold >> 6u);
		}
		else
		{
			My_CapSense_1_hysteresis[widget] = 0u;
		}
	#endif /* (0u != My_CapSense_1_FLEXIBLE_THRESHOLDS_EN)  */
}

/*******************************************************************************
* Function Name: My_CapSense_1_SetAnalogSwitchesSrcDirect
********************************************************************************
*
* Summary:
*  This API switches the charge clock source to prescaler. 
*
* Parameters:
*  None
*
* Return:
*  None
*
* Global Variables:
*  None
*
* Side Effects:
*  None
*
*******************************************************************************/
void My_CapSense_1_SetAnalogSwitchesSrcDirect(void)
{
	My_CapSense_1_CSD_CFG_REG &= ~(My_CapSense_1_CSD_CFG_PRS_SELECT);
}


/*******************************************************************************
* Function Name: My_CapSense_1_DisableBaselineIDAC
********************************************************************************
*
* Summary:
*  The API disables the Compensation IDAC. 
*
* Parameters:
*  None
*
* Return:
*  None
*
* Global Variables:
*  None
*
* Side Effects:
*  None
*
*******************************************************************************/
void My_CapSense_1_DisableBaselineIDAC(void)
{
	uint32 newRegValue;
    
    newRegValue = My_CapSense_1_CSD_IDAC_REG;
    
    newRegValue &= ~(My_CapSense_1_CSD_IDAC1_MODE_MASK | My_CapSense_1_CSD_IDAC2_MODE_MASK);
    newRegValue |= My_CapSense_1_CSD_IDAC1_MODE_VARIABLE;
    
	My_CapSense_1_CSD_IDAC_REG = newRegValue;
}


/*******************************************************************************
* Function Name: My_CapSense_1_EnableBaselineIDAC
********************************************************************************
*
* Summary:
*  The API enables the Compensation IDAC. 
*
* Parameters:
*  None
*
* Return:
*  None
*
* Global Variables:
*  My_CapSense_1_compensationIDAC[] - used to store a 7-bit idac value for all the sensors.
*
* Side Effects:
*  None
*
*******************************************************************************/
void  My_CapSense_1_EnableBaselineIDAC(const My_CapSense_1_CONFIG_TYPE_P4_v2_10 *config)
{
    uint32 curSensor;
    uint32 newRegValue;

    for(curSensor = 0u; curSensor < config->totalScanslotsNum; curSensor++)
    {
		My_CapSense_1_compensationIDAC[curSensor] = My_CapSense_1_modulationIDAC[curSensor] / 2u;
	    My_CapSense_1_modulationIDAC[curSensor] = (My_CapSense_1_modulationIDAC[curSensor] + 1u) / 2u;
    }
	
	CyIntDisable(My_CapSense_1_ISR_NUMBER);
	
	newRegValue = My_CapSense_1_CSD_IDAC_REG;
    newRegValue &= ~(My_CapSense_1_CSD_IDAC1_MODE_MASK | My_CapSense_1_CSD_IDAC2_MODE_MASK);
    newRegValue |= (My_CapSense_1_CSD_IDAC1_MODE_VARIABLE | My_CapSense_1_CSD_IDAC2_MODE_FIXED);
    My_CapSense_1_CSD_IDAC_REG = newRegValue;
	
	CyIntEnable(My_CapSense_1_ISR_NUMBER);
}


/*******************************************************************************
* Function Name: My_CapSense_1_SetSensitivity
********************************************************************************
*
* Summary:
*  This API sets the sensitivity value for the sensor. The sensitivity 
*  value is used during the auto-tuning algorithm executed as part of the CapSense_Start API.
*  This API is called by the application layer prior to calling the CapSense_Start API. 
*  Calling this API after execution of CapSense_Start API has no effect. 
*
* Parameters: 
*  sensor  Sensor index 
*  data    Sensitivity of the sensor. Possible values are below
*  1 - 0.1pf sensitivity
*  2 - 0.2pf sensitivity
*  3 - 0.3pf sensitivity
*  4 - 0.4pf sensitivity
*  All other values, set sensitivity to 0.4pf
*
*  Return Value:
*   None
*
* Global Variables:
* SMS_LIB_SensorSensitivity[] - This 8-bits array contains Sensitivity in the LSB nibble
*                                and Noise Reset Counter value in the MSB nibble.      
*
* Side Effects: 
*  None
*
*******************************************************************************/
void My_CapSense_1_SetSensitivity(uint32 sensor, uint32 value)
{
	if(value > 4u)
	{
		value = 4u;
	}
	
    /* Clear SensorSensitivity value in LSB nibble */
	SMS_LIB_SensorSensitivity[sensor] &= (uint8)~My_CapSense_1_SENSITIVITY_MASK;
	/* Set SensorSensitivity value in LSB nibble */
	SMS_LIB_SensorSensitivity[sensor] |= (uint8)value;
}


/*******************************************************************************
* Function Name: My_CapSense_1_GetSensitivityCoefficient
********************************************************************************
*
* Summary:
*  This API returns the K coefficient for the appropriate sensor. 
*
* Parameters: 
*  sensor:  Sensor index 
*
*  Return Value:
*   K - value for the appropriate sensor.
*
* Global Variables:
*  kValueTbl[] - This 8-bits array contains the K value for each sensor.
*
* Side Effects: 
*  None
*
*******************************************************************************/
uint8 My_CapSense_1_GetSensitivityCoefficient(uint32 sensor)
{
	return (kValueTbl[sensor]);
}


/*******************************************************************************
* Function Name: My_CapSense_1_GetNoiseEnvelope
********************************************************************************
*
* Summary:
*  This function returns the noise value of a
*  component.
*
* Parameters:
*  sensor - Sensor number.
*
* Return:
*  The noise envelope value of the sensor
*  indicated by argument.
*
* Global Variables:
*  baNoiseEnvelope[] - array with Noise Envelope.
*
* Side Effects:
*  None
*
*******************************************************************************/
uint16 My_CapSense_1_GetNoiseEnvelope(uint32 sensor)
{
    return((uint16)((uint16)noiseEnvelopeTbl[sensor] << 1u) + 1u);
}


/*******************************************************************************
* Function Name: My_CapSense_1_GetNormalizedDiffCountData
********************************************************************************
*
* Summary:
*  This API returns normalized difference count data.
*
* Parameters:
*  sensor:  Sensor number.
*
* Return:
*  This API returns a normalized count value of the sensor indicated by the 
*  argument. 
*
* Global Variables:
*  My_CapSense_1_sensorSignal[] - array with difference counts for sensors
* 
* Side Effects:
*  None
*
*******************************************************************************/
uint16 My_CapSense_1_GetNormalizedDiffCountData(uint32 sensor)
{
	return (uint16)(((uint32)My_CapSense_1_sensorSignal[sensor] << 7u) / kValueTbl[sensor]);
}

#endif /* (My_CapSense_1_TUNING_METHOD == My_CapSense_1__TUNING_AUTO) */

/* [] END OF FILE */
