/*******************************************************************************
* File Name: My_CapSense_1_SMS.h
* Version 2.10
*
* Description:
*  This file provides the declarations of the wrapper between the CapSense CSD component 
*  and Auto Tuning library.
*
* Note:
*
********************************************************************************
* Copyright 2014, Cypress Semiconductor Corporation.  All rights reserved.
* You may use this file only in accordance with the license, terms, conditions,
* disclaimers, and limitations in the end user license agreement accompanying
* the software package with which this file was provided.
*******************************************************************************/
#if !defined(CY_CAPSENSE_CSD_SMS_My_CapSense_1_H)
#define CY_CAPSENSE_CSD_SMS_My_CapSense_1_H

#include "My_CapSense_1.h"
#include "My_CapSense_1_PVT.h"

#if(0u != My_CapSense_1_CSHL_API_GENERATE)
	#include "My_CapSense_1_CSHL.h"
#endif /* (0u != My_CapSense_1_CSHL_API_GENERATE) */

#define My_CapSense_1_PRESCALERS_TBL_SIZE		(16u)

#define My_CapSense_1_CALIBRATION_FREQ_KHZ		(3000u)
#define My_CapSense_1_CALIBRATION_MD  		    (2u)

#define My_CapSense_1_MAX_RESOLUTION         (65535u)
#define My_CapSense_1_CALIBRATION_RESOLUTION	(0x0FFF0000u)

#define My_CapSense_1_PRS_CORRECTION_DIVIDER (4u)
#define My_CapSense_1_SENSITIVITY_DIVIDER    (10u)

#define My_CapSense_1_SENSITIVITY_MASK        	(0x0Fu)
#define My_CapSense_1_FLEXIBLE_THRESHOLDS_EN  	(0u)
#define My_CapSense_1_CAL_RAW_COUNT           	(3482u)

#define My_CapSense_1_THRESHOLD_1                (220u)
#define My_CapSense_1_THRESHOLD_2                (237u)
#define My_CapSense_1_THRESHOLD_3                (245u)
#define My_CapSense_1_THRESHOLD_4                (250u)


#if (0u != My_CapSense_1_IS_M0S8PERI_BLOCK)
	#define My_CapSense_1_CALIBRATION_ASD		(CYDEV_BCLK__HFCLK__KHZ /\
													 My_CapSense_1_CALIBRATION_FREQ_KHZ)
#else
	#define My_CapSense_1_MIN_IMO_FREQ_KHZ		(6000u)
	#if(CYDEV_BCLK__HFCLK__KHZ < My_CapSense_1_MIN_IMO_FREQ_KHZ)
		#define My_CapSense_1_CALIBRATION_ASD			(1u)
	#else
		#define My_CapSense_1_CALIBRATION_ASD	(CYDEV_BCLK__HFCLK__KHZ / My_CapSense_1_CALIBRATION_MD /\
												 	 My_CapSense_1_CALIBRATION_FREQ_KHZ)
	#endif /* (CYDEV_BCLK__HFCLK__KHZ < My_CapSense_1_MIN_IMO_FREQ_KHZ) */
#endif /* (0u != My_CapSense_1_IS_M0S8PERI_BLOCK) */

#if (0u != My_CapSense_1_IS_M0S8PERI_BLOCK)
    /*  Dividers are not chained */ 
	#if(My_CapSense_1_PRS_OPTIONS != My_CapSense_1__PRS_NONE)
		#define My_CapSense_1_SENSETIVITY_FACTOR (1228u)
	#else
		#define My_CapSense_1_SENSETIVITY_FACTOR (2456u)
	#endif /* (My_CapSense_1_PRS_OPTIONS == My_CapSense_1__PRS_NONE) */
#else
    /*  Dividers are chained */
	#if(My_CapSense_1_PRS_OPTIONS != My_CapSense_1__PRS_NONE)
		#define My_CapSense_1_SENSETIVITY_FACTOR (1228u/My_CapSense_1_CALIBRATION_MD)
	#else
		#define My_CapSense_1_SENSETIVITY_FACTOR (2456u/My_CapSense_1_CALIBRATION_MD)
	#endif /* (My_CapSense_1_PRS_OPTIONS == My_CapSense_1__PRS_NONE) */
#endif /* (0u != My_CapSense_1_IS_M0S8PERI_BLOCK) */


/***************************************
*        Function Prototypes
***************************************/
void My_CapSense_1_AutoTune(void);
void My_CapSense_1_UpdateThresholds(uint32 sensor);
void My_CapSense_1_SetSensitivity(uint32 sensor, uint32 value);
void My_CapSense_1_SetAnalogSwitchesSrcDirect(void);
uint16 My_CapSense_1_GetNoiseEnvelope(uint32 sensor);
uint8 My_CapSense_1_GetSensitivityCoefficient(uint32 sensor);
uint16 My_CapSense_1_GetNormalizedDiffCountData(uint32 sensor);


/***************************************
*     Vars with External Linkage
***************************************/

/*  Real Charge Divider values */
#if (0u != My_CapSense_1_IS_M0S8PERI_BLOCK)
    /*  Dividers are not chained */
	#if (CYDEV_BCLK__HFCLK__MHZ >= 12u)
		extern const uint8 prescalerTable[My_CapSense_1_PRESCALERS_TBL_SIZE];
	#elif (CYDEV_BCLK__HFCLK__MHZ >= 6u)
		extern const uint8 prescalerTable[My_CapSense_1_PRESCALERS_TBL_SIZE];
	#else
		extern const uint8 prescalerTable[My_CapSense_1_PRESCALERS_TBL_SIZE];
	#endif /* (CYDEV_BCLK__HFCLK__MHZ > 12u) */
#else
    /*  Dividers are chained */ 
	#if (CYDEV_BCLK__HFCLK__MHZ > 24u)
		extern const uint8 prescalerTable[My_CapSense_1_PRESCALERS_TBL_SIZE];
	#elif (CYDEV_BCLK__HFCLK__MHZ >12u)
		extern const uint8 prescalerTable[My_CapSense_1_PRESCALERS_TBL_SIZE];
	#else   /* (CYDEV_BCLK__HFCLK__MHZ > 12u) */
		extern const uint8 prescalerTable[My_CapSense_1_PRESCALERS_TBL_SIZE];
	#endif /* (CYDEV_BCLK__HFCLK__MHZ > 24u) */
#endif /* (0u != My_CapSense_1_IS_M0S8PERI_BLOCK) */


extern uint8 noiseEnvelopeTbl[My_CapSense_1_TOTAL_SENSOR_COUNT];
extern uint8 runningDifferenceTbl[My_CapSense_1_TOTAL_SENSOR_COUNT];
extern uint8 signRegisterTbl[My_CapSense_1_TOTAL_SENSOR_COUNT];
extern uint16 sampleMinTbl[My_CapSense_1_TOTAL_SENSOR_COUNT];
extern uint16 sampleMaxTbl[My_CapSense_1_TOTAL_SENSOR_COUNT];
extern uint16 previousSampleTbl[My_CapSense_1_TOTAL_SENSOR_COUNT];
extern uint8 kValueTbl[My_CapSense_1_TOTAL_SENSOR_COUNT];


#endif  /* (CY_CAPSENSE_CSD_SMS_My_CapSense_1_H) */

/* [] END OF FILE */
