/*******************************************************************************
* File Name: My_CapSense_1_Sns.c  
* Version 2.10
*
* Description:
*  This file contains API to enable firmware control of a Pins component.
*
* Note:
*
********************************************************************************
* Copyright 2008-2014, Cypress Semiconductor Corporation.  All rights reserved.
* You may use this file only in accordance with the license, terms, conditions, 
* disclaimers, and limitations in the end user license agreement accompanying 
* the software package with which this file was provided.
*******************************************************************************/

#include "cytypes.h"
#include "My_CapSense_1_Sns.h"

#define SetP4PinDriveMode(shift, mode)  \
    do { \
        My_CapSense_1_Sns_PC =   (My_CapSense_1_Sns_PC & \
                                (uint32)(~(uint32)(My_CapSense_1_Sns_DRIVE_MODE_IND_MASK << (My_CapSense_1_Sns_DRIVE_MODE_BITS * (shift))))) | \
                                (uint32)((uint32)(mode) << (My_CapSense_1_Sns_DRIVE_MODE_BITS * (shift))); \
    } while (0)


/*******************************************************************************
* Function Name: My_CapSense_1_Sns_Write
********************************************************************************
*
* Summary:
*  Assign a new value to the digital port's data output register.  
*
* Parameters:  
*  prtValue:  The value to be assigned to the Digital Port. 
*
* Return: 
*  None 
*  
*******************************************************************************/
void My_CapSense_1_Sns_Write(uint8 value) 
{
    uint8 drVal = (uint8)(My_CapSense_1_Sns_DR & (uint8)(~My_CapSense_1_Sns_MASK));
    drVal = (drVal | ((uint8)(value << My_CapSense_1_Sns_SHIFT) & My_CapSense_1_Sns_MASK));
    My_CapSense_1_Sns_DR = (uint32)drVal;
}


/*******************************************************************************
* Function Name: My_CapSense_1_Sns_SetDriveMode
********************************************************************************
*
* Summary:
*  Change the drive mode on the pins of the port.
* 
* Parameters:  
*  mode:  Change the pins to one of the following drive modes.
*
*  My_CapSense_1_Sns_DM_STRONG     Strong Drive 
*  My_CapSense_1_Sns_DM_OD_HI      Open Drain, Drives High 
*  My_CapSense_1_Sns_DM_OD_LO      Open Drain, Drives Low 
*  My_CapSense_1_Sns_DM_RES_UP     Resistive Pull Up 
*  My_CapSense_1_Sns_DM_RES_DWN    Resistive Pull Down 
*  My_CapSense_1_Sns_DM_RES_UPDWN  Resistive Pull Up/Down 
*  My_CapSense_1_Sns_DM_DIG_HIZ    High Impedance Digital 
*  My_CapSense_1_Sns_DM_ALG_HIZ    High Impedance Analog 
*
* Return: 
*  None
*
*******************************************************************************/
void My_CapSense_1_Sns_SetDriveMode(uint8 mode) 
{
	SetP4PinDriveMode(My_CapSense_1_Sns__0__SHIFT, mode);
}


/*******************************************************************************
* Function Name: My_CapSense_1_Sns_Read
********************************************************************************
*
* Summary:
*  Read the current value on the pins of the Digital Port in right justified 
*  form.
*
* Parameters:  
*  None 
*
* Return: 
*  Returns the current value of the Digital Port as a right justified number
*  
* Note:
*  Macro My_CapSense_1_Sns_ReadPS calls this function. 
*  
*******************************************************************************/
uint8 My_CapSense_1_Sns_Read(void) 
{
    return (uint8)((My_CapSense_1_Sns_PS & My_CapSense_1_Sns_MASK) >> My_CapSense_1_Sns_SHIFT);
}


/*******************************************************************************
* Function Name: My_CapSense_1_Sns_ReadDataReg
********************************************************************************
*
* Summary:
*  Read the current value assigned to a Digital Port's data output register
*
* Parameters:  
*  None 
*
* Return: 
*  Returns the current value assigned to the Digital Port's data output register
*  
*******************************************************************************/
uint8 My_CapSense_1_Sns_ReadDataReg(void) 
{
    return (uint8)((My_CapSense_1_Sns_DR & My_CapSense_1_Sns_MASK) >> My_CapSense_1_Sns_SHIFT);
}


/* If Interrupts Are Enabled for this Pins component */ 
#if defined(My_CapSense_1_Sns_INTSTAT) 

    /*******************************************************************************
    * Function Name: My_CapSense_1_Sns_ClearInterrupt
    ********************************************************************************
    *
    * Summary:
    *  Clears any active interrupts attached to port and returns the value of the 
    *  interrupt status register.
    *
    * Parameters:  
    *  None 
    *
    * Return: 
    *  Returns the value of the interrupt status register
    *  
    *******************************************************************************/
    uint8 My_CapSense_1_Sns_ClearInterrupt(void) 
    {
		uint8 maskedStatus = (uint8)(My_CapSense_1_Sns_INTSTAT & My_CapSense_1_Sns_MASK);
		My_CapSense_1_Sns_INTSTAT = maskedStatus;
        return maskedStatus >> My_CapSense_1_Sns_SHIFT;
    }

#endif /* If Interrupts Are Enabled for this Pins component */ 


/* [] END OF FILE */
