/*******************************************************************************
* File Name: My_Mux_PM.c
* Version 1.80
*
* Description:
*  This file contains the setup, control, and status commands to support 
*  the component operation in the low power mode. 
*
* Note:
*
********************************************************************************
* Copyright 2015, Cypress Semiconductor Corporation.  All rights reserved.
* You may use this file only in accordance with the license, terms, conditions, 
* disclaimers, and limitations in the end user license agreement accompanying 
* the software package with which this file was provided.
*******************************************************************************/

#include "My_Mux.h"

/* Check for removal by optimization */
#if !defined(My_Mux_Sync_ctrl_reg__REMOVED)

static My_Mux_BACKUP_STRUCT  My_Mux_backup = {0u};

    
/*******************************************************************************
* Function Name: My_Mux_SaveConfig
********************************************************************************
*
* Summary:
*  Saves the control register value.
*
* Parameters:
*  None
*
* Return:
*  None
*
*******************************************************************************/
void My_Mux_SaveConfig(void) 
{
    My_Mux_backup.controlState = My_Mux_Control;
}


/*******************************************************************************
* Function Name: My_Mux_RestoreConfig
********************************************************************************
*
* Summary:
*  Restores the control register value.
*
* Parameters:
*  None
*
* Return:
*  None
*
*
*******************************************************************************/
void My_Mux_RestoreConfig(void) 
{
     My_Mux_Control = My_Mux_backup.controlState;
}


/*******************************************************************************
* Function Name: My_Mux_Sleep
********************************************************************************
*
* Summary:
*  Prepares the component for entering the low power mode.
*
* Parameters:
*  None
*
* Return:
*  None
*
*******************************************************************************/
void My_Mux_Sleep(void) 
{
    My_Mux_SaveConfig();
}


/*******************************************************************************
* Function Name: My_Mux_Wakeup
********************************************************************************
*
* Summary:
*  Restores the component after waking up from the low power mode.
*
* Parameters:
*  None
*
* Return:
*  None
*
*******************************************************************************/
void My_Mux_Wakeup(void)  
{
    My_Mux_RestoreConfig();
}

#endif /* End check for removal by optimization */


/* [] END OF FILE */
